@@ -65,6 +65,8 @@ struct reg
 };
 
 extern void make_syscall(void);
+extern void fork_symtab_file(void);
+extern void exec_symtab_file(void);
 int check_ioctl(int cmd, int arg);
 int check_ipc(unsigned long arg1, unsigned long arg2);
 int check_socketcall(int call, unsigned long *args, unsigned int * const res);
@@ -95,6 +97,9 @@ void check_syscall(struct reg * const regs)
 		chkr_do_end();
 		break;
 	case SYS_fork:
+		make_syscall();
+		if (RES == 0)
+		  fork_symtab_file();
 		break;
 	case SYS_read:
 		chkr_check_addr(PTR2, ARG3, CHKR_TW);
@@ -152,12 +157,15 @@ void check_syscall(struct reg * const regs)
 		  }
 		  chkr_check_addr(ptr, sizeof(char *), CHKR_RO);
 		}
+		chkr_do_end();
+		exec_symtab_file();
 		break;
 	case SYS_chdir:
 		chkr_check_str(PTR1, CHKR_RO);
 		break;
 	case SYS_time:
-		chkr_check_addr(PTR1, sizeof(time_t), CHKR_WO);
+		if (ARG1)
+		  chkr_check_addr(PTR1, sizeof(time_t), CHKR_WO);
 		break;
 	case SYS_mknod:
 		chkr_check_str(PTR1, CHKR_RO);
@@ -198,13 +206,11 @@ void check_syscall(struct reg * const regs)
 		error = 1;
 		break;
 	case SYS_alarm:
-		error = 1;
 		break;
 	case SYS_oldfstat:
 		error = 1;
 		break;
 	case SYS_pause:
-		error = 1;	/* replaced by setitimer/signal */
 		break;
 	case SYS_utime:
 		chkr_check_addr(PTR2, sizeof(struct utimbuf), CHKR_RW);
@@ -306,9 +312,14 @@ void check_syscall(struct reg * const regs)
 	case SYS_setsid:
 		break;
 	case SYS_sigaction:
-		chkr_check_addr(PTR2, sizeof(struct sigaction), CHKR_RO);
+#if 1
+		__sigaction(ARG1,PTR2,PTR3);
+#else
+		if (ARG2)
+		  chkr_check_addr(PTR2, sizeof(struct sigaction), CHKR_RO);
 		if (ARG3)
 		  chkr_check_addr(PTR3, sizeof(struct sigaction), CHKR_WO);
+#endif
 		break;
 	case SYS_siggetmask:
 		error = 1;	/* old syscall */
@@ -321,7 +332,6 @@ void check_syscall(struct reg * const regs)
 	case SYS_setregid:
 		break;
 	case SYS_sigsuspend:
-		chkr_check_addr(PTR1, sizeof(sigset_t), CHKR_WO);
 		break;
 	case SYS_sigpending:
 		break;
@@ -354,13 +364,20 @@ void check_syscall(struct reg * const regs)
 		chkr_check_addr(PTR2, ARG1 * sizeof(__gid_t), CHKR_RO);
 		break;
 	case SYS_select:
-		if (ARG1)
-		  chkr_check_addr(PTR1, sizeof(fd_set), CHKR_RW);
-		if (ARG2)
-		  chkr_check_addr(PTR2, sizeof(fd_set), CHKR_RW);	
-		if (ARG3)
-		  chkr_check_addr(PTR3, sizeof(fd_set), CHKR_RW);	
-		chkr_check_addr(PTR4, sizeof(struct timeval), CHKR_RW);
+		{
+		  char **ptr;
+		  
+		  chkr_check_addr(PTR1, 5 * sizeof(int*), CHKR_RO);
+		  ptr = (char**)PTR1;
+		  if (ptr[1])
+		    chkr_check_addr(ptr[1], sizeof(fd_set), CHKR_RW);
+		  if (ptr[2])
+		    chkr_check_addr(ptr[2], sizeof(fd_set), CHKR_RW);	
+		  if (ptr[3])
+		    chkr_check_addr(ptr[3], sizeof(fd_set), CHKR_RW);	
+		  if (ptr[4])
+		    chkr_check_addr(ptr[4], sizeof(struct timeval), CHKR_RW);
+		}
 		break;
 	case SYS_symlink:
 		chkr_check_str(PTR1, CHKR_RO);
@@ -373,7 +390,8 @@ void check_syscall(struct reg * const regs)
 		chkr_check_str(PTR1, CHKR_RO);
 		chkr_check_addr(PTR2, ARG3, CHKR_TW);
 		make_syscall();
-		chkr_set_right(PTR2, RES, CHKR_WO);
+		if (RES >= 0)
+		  chkr_set_right(PTR2, RES, CHKR_WO);
 		break;
 	case SYS_uselib:
 		break;		/* no check */
@@ -525,7 +543,7 @@ int check_ioctl(int cmd, int arg)
 	case TCSETAW:
 	case TCSETAF:	    	  
 	case TCSETA:
-		chkr_check_addr((__ptr_t)arg, sizeof(struct termios), CHKR_RO);
+		chkr_check_addr((__ptr_t)arg, sizeof(struct termio), CHKR_RO);
 		break;
 	case TCSBRK:
 	case TCSBRKP:	  
@@ -624,14 +642,11 @@ int check_fcntl(int cmd, int arg)
 {
 	switch(cmd)
 	{
-	 case F_DUPFD:
-	 	break;
+	 case F_DUPFD: /* arg is used as a value */
 	 case F_GETFD:
 	 case F_GETFL:
-	 	chkr_check_addr((__ptr_t)arg, sizeof(int), CHKR_WO);
-	 	break;
-	 case F_SETFD:
-	 case F_SETFL:
+	 case F_SETFD: /* arg is used as a value */
+	 case F_SETFL: /* arg is used as a value */
 	 	break;
 	 case F_GETLK:
 	 	chkr_check_addr((__ptr_t)arg, sizeof(struct flock), CHKR_WO);
@@ -640,6 +655,9 @@ int check_fcntl(int cmd, int arg)
 	 case F_SETLKW:	/* TG: what is the difference ? */
 	 	chkr_check_addr((__ptr_t)arg, sizeof(struct flock), CHKR_RO);
 	 	break;
+	 case F_SETOWN:  /* arg is used as a value */
+	 case F_GETOWN:
+	 	break;
 	 default:
 	  	chkr_header(M_FCNTL_UNIMPLEMENT);
   		chkr_printf(M_SEND_DESCRIPTION);
@@ -654,10 +672,10 @@ int check_socketcall(int call, unsigned long *args, unsigned int * const res)
   switch(call)
   {
 	case SYS_ACCEPT:
-		chkr_check_addr((__ptr_t)args[3], sizeof (int), CHKR_RO);
-		chkr_check_addr((__ptr_t)args[2], *(int*)args[3], CHKR_TW);
+		chkr_check_addr((__ptr_t)args[2], sizeof (int), CHKR_RO);
+		chkr_check_addr((__ptr_t)args[1], *(int*)args[2], CHKR_TW);
 		make_syscall();
-		chkr_set_right((__ptr_t)args[2], *(int*)args[3], CHKR_RW);
+		chkr_set_right((__ptr_t)args[1], *(int*)args[2], CHKR_RW);
 		break;
 	case SYS_BIND:
 		chkr_check_addr((__ptr_t)args[1], args[3], CHKR_RO);
