@@ -28,12 +28,18 @@ Cambridge, MA 02139, USA.
 __ptr_t
 calloc (size_t nmemb, size_t size)
 {
-  register __ptr_t result = malloc (nmemb * size);
-
+  register __ptr_t result;
+  
+#ifdef CHKR_STACKBITMAP
+  chkr_check_addr(&nmemb, sizeof(nmemb), CHKR_RO);
+  chkr_check_addr(&size, sizeof(size_t), CHKR_RO);
+#endif  
+  
+  result = _malloc (nmemb * size);
   if (result != NULL)
   {
     (void) memset (result, 0, nmemb * size);
-#ifdef CHKR_BITMAP
+#ifdef CHKR_USE_BITMAP
     chkr_set_right (result, nmemb *size, CHKR_RW);
 #endif
   }        
