@@ -61,7 +61,7 @@ typedef struct
   uchar *bmbase;	/* base of the bitmap */
   MapType type;		/* type of the segment */
   const char *name;		/* name of the segment */
-  void (*efunc)(__ptr_t ptr);	/* function to call in case of error */
+  void (*efunc)(const __ptr_t ptr);	/* function to call in case of error */
 #ifdef CHKR_PROFILE
   uint addressed;
 #endif
@@ -95,13 +95,13 @@ static int bytes_per_state = 1;	/* Number of bytes regrouped by a state */
 __ptr_t known_stack_limit;	/* current %esp known */
 __ptr_t stack_bitmapped;	/* limit of the bitmap */
 
-static void chkr_access_error(__ptr_t ptr, int arq, struct BitMem *bm, char val);
-static void adjust_bm();
+static void chkr_access_error(const __ptr_t ptr, int arq, struct BitMem *bm, char val, int size);
+static void adjust_bm(void);
 #ifdef CHKR_STACKBITMAP
 void adjust_stackbitmap(__ptr_t ptr);
-static void chkr_access_error_stack(__ptr_t ptr);
+static void chkr_access_error_stack(const __ptr_t ptr);
 #endif /* CHKR_STACKBITMAP */
-static void chkr_access_error_heap (__ptr_t ptr);
+static void chkr_access_error_heap (const __ptr_t ptr);
 
 /* Optimization: what is the condition to call adjust_bm */
 #ifdef CHKR_STACKBITMAP
@@ -114,9 +114,11 @@ static void chkr_access_error_heap (__ptr_t ptr);
 uint adjust_bm_called = 0;
 #endif
 
+int __chkr_maccess = 0;	/* set to 1 when compiled with checker */
+
 /* Get information about addr in bitmem: which bitmap, which offset... */
 static void
-SetBitMem(__ptr_t addr, struct BitMem *bitmem)
+SetBitMem(const __ptr_t addr, struct BitMem *bitmem)
 {
  register MapInfo *tmpinfo = Fmapinfo;
  
@@ -159,7 +161,7 @@ SetBitMem(__ptr_t addr, struct BitMem *bitmem)
 
 /* Adjust all bitmaps (Useful for the stack) */
 static void
-adjust_bm()
+adjust_bm(void)
 {
 #ifdef CHKR_STACKBITMAP
  if (1)	/* WAS: stack_bitmapped > known_stack_limit */
@@ -204,7 +206,7 @@ adjust_bm()
  * Use this function, only when debugging Checker.
  */
 void
-chkr_disp_right(__ptr_t ptr, int len)
+chkr_disp_right(const __ptr_t ptr, int len)
 {
  struct BitMem bitmem;
  int bm_offset;
@@ -286,7 +288,7 @@ chkr_disp_right(__ptr_t ptr, int len)
 
 /* Set the right of a zone of length len, starting at ptr */
 void
-chkr_set_right(__ptr_t ptr, int len, int right)
+chkr_set_right(const __ptr_t ptr, int len, int right)
 {
  struct BitMem bitmem;
  uchar *base;	/* Bitmap base */
@@ -428,12 +430,13 @@ chkr_set_right(__ptr_t ptr, int len, int right)
  *  d    +-----------+-----------+-------------+-------------+-------------+
  */
 void
-chkr_check_addr(__ptr_t ptr, int len, int right1)
+chkr_check_addr(const __ptr_t ptr, int len, int right1)
 {
  struct BitMem bitmem;
  uchar *base;		/* Bitmap base */
  uchar val = 0;		/* Val to set in the bitmap */
  uchar mask = 0;
+ int size = len;	/* save */
  int right;			/* Good Value of right1 */
  static uchar tab_beg_offset[4] = { 0x00, 0x54, 0x50, 0x40 }; 
  static uchar tab_end_offset[4] = { 0x00, 0x01, 0x05, 0x15 };  
@@ -479,7 +482,7 @@ chkr_check_addr(__ptr_t ptr, int len, int right1)
            }
          if ((base[bitmem.bm_offset] & val) != val)
            {
-             chkr_access_error(ptr, right, &bitmem, val);
+             chkr_access_error(ptr, right, &bitmem, val, size);
              return;	/* check fails */
            }
          if (right1 == CHKR_WO)
@@ -499,7 +502,7 @@ chkr_check_addr(__ptr_t ptr, int len, int right1)
            {
              if ((base[bitmem.bm_offset] & val) != val)
                {
-                  chkr_access_error(ptr, right, &bitmem, val);
+                  chkr_access_error(ptr, right, &bitmem, val, size);
                   return;	/* check fails */
                }
              if (right1 == CHKR_WO)
@@ -515,7 +518,7 @@ chkr_check_addr(__ptr_t ptr, int len, int right1)
      mask = tab_end_offset[len] * CHKR_RW;
      if ((base[bitmem.bm_offset] & val) != val)
        {
-         chkr_access_error(ptr, right, &bitmem, val);
+         chkr_access_error(ptr, right, &bitmem, val, size);
          return;	/* check fails */
        }
      if (right1 == CHKR_WO)
@@ -542,7 +545,7 @@ chkr_check_addr(__ptr_t ptr, int len, int right1)
            }
          if ((base[bitmem.bm_offset] & val) != val)
            {
-             chkr_access_error(ptr, right, &bitmem, val);
+             chkr_access_error(ptr, right, &bitmem, val, size);
              return;	/* check fails */
            }
          if (right1 == CHKR_WO)
@@ -562,7 +565,7 @@ chkr_check_addr(__ptr_t ptr, int len, int right1)
            {
              if ((base[bitmem.bm_offset] & val) != val)
                {
-                 chkr_access_error(ptr, right, &bitmem, val);
+                 chkr_access_error(ptr, right, &bitmem, val, size);
                  return;	/* check fails */
                }
              if (right1 == CHKR_WO)
@@ -578,7 +581,7 @@ chkr_check_addr(__ptr_t ptr, int len, int right1)
      mask = tab_end_off1[len] * CHKR_RW;
      if ((base[bitmem.bm_offset] & val) != val)
        {
-         chkr_access_error(ptr, right, &bitmem, val);
+         chkr_access_error(ptr, right, &bitmem, val, size);
          return;	/* check fails */
        }
      if (right1 == CHKR_WO)
@@ -587,13 +590,13 @@ chkr_check_addr(__ptr_t ptr, int len, int right1)
      
    case SegFinish:
    case SegVoid:
-     chkr_access_error(ptr, right, &bitmem, 0);
+     chkr_access_error(ptr, right, &bitmem, 0, size);
      return;
      
    case SegText:
      if (!is_text_writable)
        if (right != CHKR_RO)
-         chkr_access_error(ptr, right, &bitmem, 0);
+         chkr_access_error(ptr, right, &bitmem, 0, size);
      return;
      
    case SegRW:
@@ -609,7 +612,7 @@ chkr_check_addr(__ptr_t ptr, int len, int right1)
  * Is this implementation poor ?
  */
 void
-chkr_check_str(__ptr_t ptr, int right)
+chkr_check_str(const __ptr_t ptr, int right)
 {
    chkr_check_addr(ptr, strlen(ptr), right);
 }
@@ -674,7 +677,7 @@ display_profile()
 
 /* initialize the informations for the bitmaps */
 static void
-init_mapinfos()
+init_mapinfos(void)
 {
  int i=0;
 
@@ -752,7 +755,7 @@ disp_block_history(__ptr_t *ptr)
 /* Code that handle access error on the stack
  */
 static void
-chkr_access_error_stack(__ptr_t ptr)
+chkr_access_error_stack(const __ptr_t ptr)
 {
  chkr_printf(M_KNOW_STACK_LIM, known_stack_limit);
  chkr_printf(M_STACK_BITMAPED, stack_bitmapped);
@@ -764,10 +767,11 @@ chkr_access_error_stack(__ptr_t ptr)
  * Display all informations.
  */
 static void
-chkr_access_error_heap (__ptr_t ptr)
+chkr_access_error_heap (const __ptr_t ptr)
 {
   struct malloc_header *block;
   __ptr_t *ptr_on_block;
+  int block_size;
 
   if (_lastblock == NULL_HEADER)
     return;
@@ -783,22 +787,25 @@ chkr_access_error_heap (__ptr_t ptr)
       return;	/* pointer on free block */
     }
   
+  /* compute the size of the block */
+  if (block->state == MDAGED)
+    block_size = block->size - be_red_zone - af_red_zone - block->s_diff;
+  else
+    block_size = block->info.busy.real_size;
+    
   /* display size of the block and its beginning */
-  chkr_printf(M_BLOCK_ID, block->state == MDAGED
-  			      ? block->size - be_red_zone - af_red_zone 
-  			      : block->info.busy.real_size, 
-  		          (uint)block + HEADER_SIZE + be_red_zone);
-  		          
+  chkr_printf(M_BLOCK_ID, block_size, (uint)block + HEADER_SIZE + be_red_zone);
+  
   if (ptr < ((__ptr_t)block + be_red_zone + HEADER_SIZE))
     chkr_printf(M_BEFORE_BLOCK,
 		(int)((int)block + be_red_zone + HEADER_SIZE - (int)ptr));
-  else if (ptr < ((__ptr_t)block + be_red_zone + HEADER_SIZE + block->info.busy.real_size))
+  else if (ptr < ((__ptr_t)block + be_red_zone + HEADER_SIZE + block_size))
     chkr_printf(M_INSIDE_BLOCK,
 		(int)(ptr) - (int)((int)block + be_red_zone + HEADER_SIZE));
   else
     chkr_printf(M_AFTER_BLOCK,
     		(int)(ptr) -
-    		(int)((__ptr_t)block + be_red_zone + HEADER_SIZE + block->info.busy.real_size));
+    		(int)((__ptr_t)block + be_red_zone + HEADER_SIZE + block_size));
 #if CHKR_SAVESTACK
   ptr_on_block = (__ptr_t*)((int)block + block->size + HEADER_SIZE - af_red_zone);
   chkr_load_symtab();
@@ -824,14 +831,25 @@ static char *right_name[]={
  * discovered. It calls chkr_access_error_*
  */
 static void
-chkr_access_error(__ptr_t ptr, int arq, struct BitMem *bm, char val)
+chkr_access_error(const __ptr_t ptr, int arq, struct BitMem *bm, char val, int size)
 {
 #ifdef CHKR_SAVESTACK
  int must_remove = 0;
 #endif /* CHKR_SAVESTACK */ 
+ int i;
+ 
+ /* without inserted code */
+ if (__chkr_maccess == 0)
+   return;
+   
+ /* Check for disabled ip (--disable= option) */
+ for (i = 0; i < disable_cur; i++)
+   if ((int)chkr_frames_ip >= (*disable)[i].first
+       && (int)chkr_frames_ip <= (*disable)[i].last)
+     return;
  
  chkr_header(M_MEM_ACCESS_ERR);
- chkr_printf(M_ACC_ERR_WHERE, right_name[arq], ptr, bm->info->name);
+ chkr_printf(M_ACC_ERR_WHERE, right_name[arq], size, ptr, bm->info->name);
  if (bm->info == mapinfo[NULLBM]) 
    {
      chkr_printf(M_USE_NULL_PTR);
