@@ -228,8 +228,10 @@ const relax_typeS md_relax_table[] =
 /* option flags */
 int checkerflag;	/* bits: 0-1 set to enable checker
 		         2   set to enable check on stack
-		         3   set to enable verbose */
-int postchecker;	/* 1 if we must insert code *after* the code */
+		         3   set to enable verbose
+		         4   set to enable special case for pushl/popl %ebp 
+		         5   set to enable check on call */
+static int postchecker;	/* 1 if we must insert code *after* the code */
 #endif
 
 static char *output_invalid PARAMS ((int c));
@@ -662,35 +664,41 @@ reloc (size, pcrel)
 #endif
 
 #ifdef CHECKER
-static void do_postchecker()
+static void
+do_chkr_emit_call(char *func, int verbose)
 {
   symbolS *symbolP;	/* points to symbol */
   expressionS expressionP;
   char *p;
-  char *name = "chkr_0_5_0_0_chkr";
- 
-  /* clear the flag */
-  postchecker = 0;
-  
-  /* verbose */
-  if (checkerflag & 8)
-    as_tsktsk("Checker post adds \tcall %s",name);
   
+  if ((checkerflag & CHKR_VERBOSE) && verbose == 1)
+    as_tsktsk("Checker pre adds \tcall %s", func);
+  if ((checkerflag & CHKR_VERBOSE) && verbose == 2)
+    as_tsktsk("Checker post adds \tcall %s", func);
+    
   /* insertion of code */
   /* Taken from gas/expr.c at line ~600 */
-  symbolP = symbol_find_or_make (name);
+  symbolP = symbol_find_or_make (func);
   p = frag_more (1 + 4);
   p[0] = CALL_PC_RELATIVE;	/* Call opcode */
   expressionP.X_op = O_symbol;
   expressionP.X_add_symbol = symbolP;
   expressionP.X_add_number = 0;
-/*  fix_new (frag_now, &p[1] - frag_now->fr_literal, 4,
-		symbolP, NULL, 0, 1, NO_RELOC); */
   fix_new_exp (frag_now, &p[1] - frag_now->fr_literal, 4,
-		&expressionP, 1, reloc (4, 1));		
+		&expressionP, 1, reloc (4, 1));
 }
 
-void static do_checker(int reverse, char *line)
+static void
+do_postchecker()
+{
+  /* clear the flag */
+  postchecker = 0;
+  
+  do_chkr_emit_call("chkr_0_5_0_0_chkr",2); 
+}
+
+void static
+do_checker(int reverse, char *line)
 {
   static int instr_to_skipf = 0;
   int j, useful;
@@ -757,7 +765,7 @@ void static do_checker(int reverse, char *line)
   else if ((i.tm.opcode_des & ODMASK) == StackOp &&
   	    !( i.operands > 0 && i.types[0] & BaseIndex))
   {
-    if(!(checkerflag & 4))
+    if(!(checkerflag & CHKR_ON_STACK))
       return;
     addrmod = 5;
     size = 4;	/* store/load only words */
@@ -777,6 +785,22 @@ void static do_checker(int reverse, char *line)
        	default:
           if (i.operands != 1)
             as_fatal("Internal error in do_checker");
+          else
+            if ((checkerflag & CHKR_FRAME_P) &&
+                 (i.types[i.operands - 1] & Reg32) && 
+                 i.regs[i.operands - 1]->reg_num == EBP_REG_NUM)
+              {
+                if (i.tm.opcode_des == PopOp)
+                  {
+                    size = 0;	/* popl %ebp: frame pointer */
+                    accmod = 0;
+                  }
+                else
+                  {
+                    do_chkr_emit_call("chkr_0_5_0_4_chkr",1);
+                    return;
+                  }
+              }
           skipf = 0;
       }
     }
@@ -798,6 +822,12 @@ void static do_checker(int reverse, char *line)
     else
       return;
   }
+  else if (i.tm.opcode_des == CallOp)
+  {
+    if (checkerflag & CHKR_ON_CALL)	/* call. BUG: lcall is not tested */
+      do_chkr_emit_call("chkr_0_7_0_4_chkr", 1);
+    return;
+  }
   else
   { 
     /* test of postcheck */
@@ -820,7 +850,10 @@ void static do_checker(int reverse, char *line)
       }
     if (!useful--)
       return;
-    addrmod = 6;
+    if ((i.tm.opcode_des & CallOp) && (checkerflag & CHKR_ON_CALL))
+      addrmod = 7;
+    else
+      addrmod = 6;
     if (i.types[useful] & BaseIndex)
       skipf = 1;	/* by %eax */
     else
@@ -893,7 +926,7 @@ void static do_checker(int reverse, char *line)
       case F80bOp:
 	accmod = i.tm.opcode_des & RWField;
 	size = 10;
-	addrmod = 2; /* ModRm */
+/*	addrmod = 2;  ModRm */
 	break;
       case StackOp:
 	accmod = (i.tm.opcode_des == PushOp) ? WriteMem : ReadMem;
@@ -1031,9 +1064,10 @@ void static do_checker(int reverse, char *line)
   }
   /* All is ready to create the name of the function */
   sprintf(name,"chkr_%1x_%1x_%1x_%1x_chkr", skipf, addrmod, accmod, size);
-  if (checkerflag & 8)
-    as_tsktsk("Checker pre adds \tcall %s",name);
-  
+
+  do_chkr_emit_call(name,1);
+      
+#if 0    
   /* insertion of code */
   /* Taken from gas/expr.c at line ~600 */
   symbolP = symbol_find_or_make (name);
@@ -1044,6 +1078,7 @@ void static do_checker(int reverse, char *line)
   p[0] = CALL_PC_RELATIVE;	/* Call opcode */
   fix_new_exp (frag_now, &p[1] - frag_now->fr_literal, 4,
 		&expressionP, 1, reloc (4, 1));
+#endif
 }
 #endif
 
@@ -1737,7 +1772,7 @@ md_assemble (line)
 	      }
 	  }
       }
-    if ((checkerflag & 3) == 3)
+    if ((checkerflag & CHKR_ENABLE) == CHKR_ENABLE)
       do_checker(found_reverse_match, line);
   }
 
@@ -2045,7 +2080,7 @@ md_assemble (line)
       }
 
 #ifdef CHECKER
-      if (postchecker && (checkerflag & 4))
+      if (postchecker && (checkerflag & CHKR_ON_STACK))
       	do_postchecker();
 #endif      	
       	
@@ -2672,14 +2707,14 @@ md_parse_option (argP, cntP, vecP)
   if (strcmp(*argP,"checker") == 0)
   {
     *argP += 7;	/* catch all the letters of 'checker' */
-    checkerflag |= 7;
+    checkerflag |= CHKR_ON_CALL | CHKR_FRAME_P | CHKR_ON_STACK | CHKR_ENABLE;
     return 1;
   }
   /* make stack checker not working */
   if (strcmp(*argP,"checker-nostack") == 0)
   {
     *argP += 15;	/* catch all the letters of 'checker-nostack' */
-    checkerflag &= ~4;
+    checkerflag &= ~CHKR_ON_STACK;
     return 1;
   }  
   /* Do not make work checker */
@@ -2693,7 +2728,7 @@ md_parse_option (argP, cntP, vecP)
   if (strcmp(*argP,"checker-verbose") == 0)
   {
     *argP += 15;	/* catch all the letters of 'checker-verbose' */
-    checkerflag |= 8;
+    checkerflag |= CHKR_VERBOSE;
     return 1;
   }
   return 0;
@@ -3054,6 +3089,16 @@ tc_aout_fix_to_chars (where, fixP, segment_address_in_file)
 	      | (((fixP->fx_pcrel << 0) & 0x01) & 0x0f));
 }
 
+#ifdef CHECKER
+void
+tc_aout_pre_write_hook(headers)
+    object_headers *headers;
+{
+ if ((checkerflag & CHKR_ENABLE) == CHKR_ENABLE)
+   H_SET_VERSION(headers, CHKR_AOUT_VERSION);
+}
+#endif
+
 #endif /* OBJ_AOUT or OBJ_BOUT */
 
 #if defined (I386COFF)
